/*==========================================================================;
 *
 *  Copyright (C) 1995-2001 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       idirect3dsurface8.cpp
 *  Content:    Direct3D capabilities include file
 *
 ***************************************************************************/
#include "d3dprj.h"

struct PixelTag 
{
	 enum _D3DFORMAT	Format;
	 int				dwSize;
	 GLenum				type;
	 GLenum				format;
};

static struct PixelTag DX_Fmt[] = {	
    { D3DFMT_R8G8B8         , 4	, GL_RGB, GL_UNSIGNED_BYTE },
    { D3DFMT_X8R8G8B8       , 4 , GL_RGBA, GL_UNSIGNED_BYTE	},
	{ D3DFMT_A8R8G8B8       , 4 , GL_RGBA, GL_UNSIGNED_BYTE	},
    { D3DFMT_X1R5G5B5       , 4 , GL_RGBA, GL_UNSIGNED_BYTE	},
	{ D3DFMT_D16_LOCKABLE    , 2	, GL_DEPTH_COMPONENT, GL_UNSIGNED_SHORT	},
	{ D3DFMT_UNKNOWN		, 0  }
};

BIDirect3DSurface8::BIDirect3DSurface8(IDirect3DDevice8 *pDevice, int Width, int Height, D3DFORMAT Format, D3DMULTISAMPLE_TYPE mstype, BOOL Lockable) :
m_pDevice(pDevice), m_Width(Width), m_Height(Height), m_Format(Format), m_mstype(mstype), m_refCount(0)
{
	MAPIDEBUG(new Surface);
	struct PixelTag * p = DX_Fmt;
	int m_Pitch = 4 * m_Width;
	while(p->Format!=D3DFMT_UNKNOWN)
	{
		if (p->Format == Format)
		{
			m_Pitch = p->dwSize * m_Width;
			break;
		}
		p++;
	}

	this->m_pLocalData = new char[m_Width * m_Pitch];
	ASSERT(m_pLocalData);
	memset(m_pLocalData, 0xAA, m_Width * m_Pitch);

}
ULONG BIDirect3DSurface8::AddRef()
{
	MAPIDEBUG(Surface::AddRef);
	m_refCount++;
	return D3D_OK;
}

HRESULT BIDirect3DSurface8::FreePrivateData(REFGUID refguid)
{
	APIDEBUG(FreePrivateData);
	return D3D_OK;
}

HRESULT BIDirect3DSurface8::GetPrivateData(REFGUID refguid, void *pData, DWORD *pSizeOfData)
{
	APIDEBUG(GetPrivateData);
	return D3D_OK;
}

HRESULT BIDirect3DSurface8::LockRect(D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags)
{	
	APIDEBUG(Surface::LockRect);
	RECT r;
	int viewport_h = m_Height;
	if (!pRect)
	{
		pRect = &r;
		r.bottom = m_Height-1;
		r.top = 0;
		r.left = 0;
		r.right = m_Width - 1;
	}	

	struct PixelTag * p = DX_Fmt;
	while(p->Format!=D3DFMT_UNKNOWN)
	{
		if (p->Format == m_Format)
		{
			pLockedRect->Pitch = p->dwSize * (pRect->right - pRect->left + 1);
			pLockedRect->pBits = (char*)m_pLocalData + (pRect->left + pRect->top * m_Width ) * p->dwSize;	
		/*	DISABLED (GL calls failure in UnlockRect ??)
			glReadPixels(	pRect->left, 
							viewport_h - pRect->bottom - 1, 
							pRect->right-pRect->left+1, 
							pRect->bottom-pRect->top-1, 
							p->type, 
							p->format, 
							m_pLocalData
			);*/
			break;
		}
		p++;
	}
	m_rectLock = *pRect;
	m_flagsLock = Flags;
	return D3D_OK;
}

HRESULT BIDirect3DSurface8::QueryInterface(REFIID riid, void **ppvObj)
{
	APIDEBUG(Surface::QueryInterface);
	*ppvObj = 0;
	return D3D_OK;
}

ULONG BIDirect3DSurface8::Release()
{	
	MAPIDEBUG(Surface::Release);
	m_refCount--;
	if (m_refCount < 0)
	{
		delete [] m_pLocalData;
		delete this;
	}	
	return D3D_OK;
}

HRESULT BIDirect3DSurface8::SetPrivateData(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags)
{
	APIDEBUG(Surface::SetPrivateData);
	return D3D_OK;
}

HRESULT BIDirect3DSurface8::UnlockRect()
{
	APIDEBUG(Surface::UnlockRect);
	RECT *pRect = &m_rectLock;
	struct PixelTag * p = DX_Fmt;
	int viewport_h = m_Height;

	while(p->Format!=D3DFMT_UNKNOWN)
	{
		if (p->Format == m_Format)
		{
			glRasterPos2i(pRect->left, viewport_h - pRect->bottom - 1);
			glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
			glDrawPixels(pRect->right-pRect->left+1, pRect->bottom-pRect->top+1, p->type, p->format, m_pLocalData);
			break;
		}
		p++;
	}	

	return D3D_OK;
}

HRESULT 
BIDirect3DSurface8::GetDevice(IDirect3DDevice8 **ppDevice)
{
	IAPIDEBUG(Surface::GetDevice);
	m_pDevice->AddRef();
	*ppDevice = m_pDevice;
	return D3D_OK;
}

HRESULT
BIDirect3DSurface8::GetContainer(REFIID iid, void **ppContainer)
{
	APIDEBUG(Surface::GetContainer);
	m_pDevice->AddRef();
	* ppContainer = m_pDevice;
	return D3D_OK;
}

HRESULT
BIDirect3DSurface8::GetDesc(D3DSURFACE_DESC *pDesc)
{
	APIDEBUG(Surface::GetDesc);
	memset(pDesc, 0, sizeof(D3DSURFACE_DESC));
	pDesc->Format = this->m_Format;
	pDesc->Height = this->m_Height;
	pDesc->Width = this->m_Width;
	pDesc->MultiSampleType = this->m_mstype;
	return D3D_OK;
}
	



/**/

