/*==========================================================================;
 *
 *  Copyright (C) 1995-2001 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       idirect3dtexture8.cpp
 *  Content:    Direct3D capabilities include file
 *
 ***************************************************************************/
#include "d3dprj.h"

GLenum D3D8ToGLEnum(D3DFORMAT fmt)
{
	return GL_RGBA;
}

class BIDirect3DSurface8_itex : public BIDirect3DSurface8
{
public:
	BIDirect3DSurface8_itex(BIDirect3DTexture8 *pTex) 
		: BIDirect3DSurface8(pTex->m_pDevice, pTex->m_Width, pTex->m_Height, pTex->m_Format, D3DMULTISAMPLE_NONE, TRUE), m_pTex(pTex)
	{
		pTex->AddRef();
	}	

	STDMETHOD(LockRect)(D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags)
	{
		return m_pTex->LockRect(0, pLockedRect, pRect, Flags);
	}

	STDMETHOD_(ULONG, Release)()
	{
		return m_pTex->Release();
	}

	STDMETHOD(UnlockRect)()
	{
		return m_pTex->UnlockRect(0);
	}

	STDMETHOD(GetContainer)(REFIID iid, void **ppContainer)
	{
		APIDEBUG(Surface_Itex::GetContainer);
		m_pTex->AddRef();
		* ppContainer = m_pTex;
		return D3D_OK;
	}

	BIDirect3DTexture8	*	m_pTex;
};

struct PixelTag 
{
	 enum _D3DFORMAT	Format;
	 int				dwSize;
	 GLenum				internalFmt;
	 GLenum				textureFmt;
};

static struct PixelTag DX_Fmt[] = {	
    { D3DFMT_R8G8B8         , 4 , GL_RGBA8 , GL_BGRA_EXT },
    { D3DFMT_A8R8G8B8       , 4 , GL_RGBA8 , GL_BGRA_EXT },
    { D3DFMT_X8R8G8B8       , 4 , GL_RGBA8 , GL_BGRA_EXT },
    { D3DFMT_R5G6B5         , 2 , GL_RGB5  , GL_BGR_EXT  },
    { D3DFMT_X1R5G5B5       , 2 , GL_RGB5  , GL_BGR_EXT },
    { D3DFMT_A1R5G5B5       , 2 , GL_RGB5  , GL_BGR_EXT },
    { D3DFMT_A4R4G4B4       , 2 , GL_RGBA4 , GL_BGRA_EXT },
    { D3DFMT_R3G3B2         , 1 , GL_R3_G3_B2 , GL_BGR_EXT	},
    { D3DFMT_A8             , 1 , GL_ALPHA8 , GL_ALPHA	},
    { D3DFMT_A8R3G3B2       , 2 , GL_RGBA4 , GL_BGRA_EXT 	},
    { D3DFMT_X4R4G4B4       , 2 , GL_RGBA4 , GL_BGRA_EXT  	},
    { D3DFMT_A2B10G10R10    , 4 , GL_RGB10_A2 , GL_BGRA_EXT 	},
    { D3DFMT_G16R16         , 4 , GL_RGBA8 , GL_BGRA_EXT 	},

    { D3DFMT_A8P8           , 2 , GL_LUMINANCE8_ALPHA8 , GL_LUMINANCE_ALPHA  	},
    { D3DFMT_P8             , 1 , GL_COLOR_INDEX , GL_COLOR_INDEX },

    { D3DFMT_L8             , 1 , GL_LUMINANCE8 , GL_LUMINANCE 	},
    { D3DFMT_A8L8           , 2 , GL_LUMINANCE8_ALPHA8 , GL_LUMINANCE 	},
    { D3DFMT_A4L4           , 1 , GL_LUMINANCE4_ALPHA4 , GL_LUMINANCE	},

    { D3DFMT_V8U8           , 2 , GL_RGBA4 , GL_BGRA_EXT	},		// GL_DU8DV8_ATI
    { D3DFMT_L6V5U5         , 2 , GL_RGBA4 , GL_BGRA_EXT  	},	
    { D3DFMT_X8L8V8U8       , 4 , GL_RGBA8 , GL_BGRA_EXT 	},
    { D3DFMT_Q8W8V8U8       , 4 , GL_RGBA8 , GL_BGRA_EXT 	},
    { D3DFMT_V16U16         , 4 , GL_RGBA8 , GL_BGRA_EXT 	},
    { D3DFMT_W11V11U10      , 4 , GL_RGBA8 , GL_BGRA_EXT 	},
    { D3DFMT_A2W10V10U10    , 4 , GL_RGBA8 , GL_BGRA_EXT 	},

    { D3DFMT_UYVY           , 4 , GL_RGBA8 , GL_BGRA_EXT 	},
    { D3DFMT_YUY2           , 4 , GL_RGBA8 , GL_BGRA_EXT 	},
#ifdef _WGLEXT_HH
    { D3DFMT_DXT1           , 4 , GL_COMPRESSED_RGB_S3TC_DXT1_EXT , GL_BGRA_EXT 	},
    { D3DFMT_DXT2           , 4 , GL_COMPRESSED_RGBA_S3TC_DXT1_EXT , GL_BGRA_EXT 	},
    { D3DFMT_DXT3           , 4 , GL_COMPRESSED_RGBA_S3TC_DXT3_EXT , GL_BGRA_EXT 	},
    { D3DFMT_DXT4           , 4 , GL_COMPRESSED_RGBA_S3TC_DXT5_EXT , GL_BGRA_EXT 	},
    { D3DFMT_DXT5           , 4 , GL_COMPRESSED_RGBA_S3TC_DXT5_EXT , GL_BGRA_EXT 	},
#endif
	{ D3DFMT_UNKNOWN		, 0 , 0 , 0 	}
};


BIDirect3DTexture8::BIDirect3DTexture8(IDirect3DDevice8 *pDevice, UINT Width, UINT Height, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool) :
m_pDevice(pDevice), m_Width(Width), m_Height(Height), m_Levels(Levels), m_Usage(Usage), m_Format(Format), m_Pool(Pool), m_refCount(0),
m_InternalFmt(0), m_TextureFmt(0), m_pLocalImage(0), m_Pitch(0)
{	
	MAPIDEBUG(new Texture);
	GL_DEBUG_TRY();

	glGenTextures(1, &m_GlHandle);	
	glBindTexture(GL_TEXTURE_2D, m_GlHandle);

	struct PixelTag * p = DX_Fmt;
	m_Pitch = 4 * m_Width;
	while(p->Format!=D3DFMT_UNKNOWN)
	{
		if (p->Format == Format)
		{
			m_InternalFmt = p->internalFmt;
			m_TextureFmt = p->textureFmt;
			m_Pitch = p->dwSize * m_Width;
			
			break;
		}
		p++;
	}
	
	m_pLocalImage = new char[m_Width * m_Pitch];
	ASSERT(m_pLocalImage);
	memset(m_pLocalImage, 0xcd, m_Width * m_Pitch);

	GL_DEBUG_CATCH();
}

	

void
BIDirect3DTexture8::BindTexture()
{
	RAPIDEBUG(BindTexture);
	GL_DEBUG_TRY();

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, m_GlHandle);

	GL_DEBUG_CATCH();
}

ULONG
BIDirect3DTexture8::AddRef()
{
	MAPIDEBUG(Texture::AddRef);
	m_refCount++;
	return 0;
}

HRESULT 
BIDirect3DTexture8::FreePrivateData(REFGUID refguid)
{
	APIDEBUG(FreePrivateData);
	return D3D_OK;
}

HRESULT
BIDirect3DTexture8::GetDevice(IDirect3DDevice8 **ppDevice)
{
	IAPIDEBUG(GetDevice);
	m_pDevice->AddRef();
	*ppDevice = m_pDevice;	
	return D3D_OK;
}

DWORD
BIDirect3DTexture8::GetLevelCount()
{
	IAPIDEBUG(GetLevelCount);
	return m_Levels;
}

DWORD
BIDirect3DTexture8::GetLOD()
{
	IAPIDEBUG(GetLOD);
	return m_Levels;
}

DWORD
BIDirect3DTexture8::GetPriority()
{
	IAPIDEBUG(GetPriority);
	return m_Priority;
}

HRESULT
BIDirect3DTexture8::GetPrivateData(REFGUID refguid, void *pData, DWORD *pSizeOfData)
{
	// TODO: Handle this com interface
	IAPIDEBUG(GetPrivateData);
	*pSizeOfData = 0;
	return D3D_OK;
}

D3DRESOURCETYPE
BIDirect3DTexture8::GetType()
{
	IAPIDEBUG(GetType);
	return D3DRTYPE_TEXTURE;
}

void
BIDirect3DTexture8::PreLoad()
{
	APIDEBUG(PreLoad);
	return ;
}

HRESULT
BIDirect3DTexture8::QueryInterface(REFIID, void **ppvObj)
{
	// TODO: Handle this com interface - should always return an error
	APIDEBUG(QueryInterface);
	*ppvObj = 0;
	return D3D_OK;
}

ULONG
BIDirect3DTexture8::Release()
{
	MAPIDEBUG(Texture::Release);
	__try{
		GL_DEBUG_TRY();
		m_refCount--;

		if ( m_refCount < 0)
		{
			MAPIDEBUG(Texture::DELETE);	
			if (m_GlHandle)
			{
				glDeleteTextures(1, &m_GlHandle);
				m_GlHandle = 0;
				delete[] m_pLocalImage;
			}
		}
		GL_DEBUG_CATCH();

	}__except(1, 1){
		APIDEBUG(EXCEPTION CATCHED);
	}
	return D3D_OK;
}

DWORD
BIDirect3DTexture8::SetPriority(DWORD PriorityNew)
{
	APIDEBUG(SetPriority);
	m_Priority = PriorityNew;	
	float v = (float)PriorityNew / 255.f;
	GL_DEBUG_TRY();
	glPrioritizeTextures(1, &m_GlHandle, &v);
	GL_DEBUG_CATCH();
	return D3D_OK;
}

HRESULT
BIDirect3DTexture8::SetPrivateData(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags)
{
	APIDEBUG(SetPrivateData);
	return D3D_OK;
}

HRESULT BIDirect3DTexture8::GetLevelDesc(UINT Level, D3DSURFACE_DESC *pDesc)
{	
	IAPIDEBUG(GetLevelDesc);
	if (Level>m_Levels)
		return D3DERR_INVALIDCALL;

	memset(pDesc, 0, sizeof(pDesc));
	pDesc->Format = m_Format;
	pDesc->Height = m_Height>>Level;
	pDesc->Width = m_Width>>Level;

	return D3D_OK;
}

HRESULT BIDirect3DTexture8::LockRect(UINT Level, D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags)
{
	APIDEBUG(Texture::LockRect);
	ASSERT(m_Pitch);
	pLockedRect->pBits = m_pLocalImage;
	pLockedRect->Pitch = m_Pitch;
	
	return D3D_OK;
}

HRESULT BIDirect3DTexture8::GetSurfaceLevel(UINT Level, IDirect3DSurface8 **ppSurfaceLevel)
{
	IAPIDEBUG(GetSurfaceLevel);
	if ((Level>m_Levels)||(!ppSurfaceLevel))
		return D3DERR_INVALIDCALL;
	if( Level == 0 )
		*ppSurfaceLevel = new BIDirect3DSurface8_itex(this);
	else
		*ppSurfaceLevel = new BIDirect3DSurface8(m_pDevice,m_Width,m_Height,m_Format,D3DMULTISAMPLE_NONE,TRUE);
	return D3D_OK; // not supported
}

HRESULT BIDirect3DTexture8::UnlockRect(UINT Level)
{
	APIDEBUG(Texture::UnlockRect);
	GL_DEBUG_TRY();

	glEnable( GL_TEXTURE_2D);
	glBindTexture( GL_TEXTURE_2D, m_GlHandle );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	glPixelStorei( GL_PACK_ROW_LENGTH, 0);
	glPixelStorei( GL_PACK_SKIP_PIXELS, 0);		

	if( m_Format != D3DFMT_A4R4G4B4 )
	glTexImage2D( GL_TEXTURE_2D,
		0, 						    // base level
		m_InternalFmt,		 		// number of color components in the texture
		m_Width, 
		m_Height,
		0, 							// border
		m_TextureFmt, 				// texture format
		GL_UNSIGNED_BYTE, 			// texture type
		m_pLocalImage
	);

	GL_DEBUG_CATCH();

	
	return D3D_OK;
}

HRESULT BIDirect3DTexture8::AddDirtyRect(CONST RECT *pDirtyRect)
{
	APIDEBUG(AddDirtyRect);
	return D3D_OK;
}

DWORD BIDirect3DTexture8::SetLOD(DWORD lod)
{	
	APIDEBUG(SetLOD);
	m_Lod = lod;
	return m_Lod;
}
