/*==========================================================================;
 *
 *  Copyright (C) 1995-2001 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       idirect3dvertexbuffer8.cpp
 *  Content:    Direct3D capabilities include file
 *
 ***************************************************************************/
#include "d3dprj.h"

extern void *glAllocateMemory(int size);
extern void  glFreeMemory(void *ptr);

BIDirect3DVertexBuffer8::BIDirect3DVertexBuffer8(IDirect3DDevice8 *pDevice, UINT Length, DWORD Usage, DWORD FVF, D3DPOOL pool) 
: m_pDevice(pDevice), m_Length(Length), m_Usage(Usage), m_FVF(FVF), m_pool(pool), m_refCount(0)
{
	APIDEBUG(new VertexBuffer);
	GL_DEBUG_TRY();
	m_pVtxBuffer = (BYTE*)glAllocateMemory(Length);
	GL_DEBUG_CATCH();
}

HRESULT BIDirect3DVertexBuffer8::FreePrivateData(REFGUID refguid)
{
	APIDEBUG(VertexBuffer::FreePrivateData);
	return D3D_OK;
}

HRESULT BIDirect3DVertexBuffer8::GetPrivateData(REFGUID refguid, void *pData, DWORD *pSizeOfData)
{
	APIDEBUG(VertexBuffer::GetPrivateData);
	return D3D_OK;
}

void BIDirect3DVertexBuffer8::PreLoad()
{
	APIDEBUG(VertexBuffer::PreLoad);
	return;
}

HRESULT BIDirect3DVertexBuffer8::QueryInterface(REFIID riid, void **ppvObj)
{
	IAPIDEBUG(VertexBuffer::QueryInterface);
	*ppvObj = 0;
	return D3D_OK;
}

ULONG BIDirect3DVertexBuffer8::Release()
{	
	MAPIDEBUG(VertexBuffer::Release);
	m_refCount--;
	if ( m_refCount < 0 )
	{
		MAPIDEBUG(VertexBuffer::DELETE);
		glFreeMemory(m_pVtxBuffer);
		delete this;
	}	
	return D3D_OK;
}

HRESULT BIDirect3DVertexBuffer8::SetPrivateData(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags)
{
	APIDEBUG(VertexBuffer::SetPrivateData);
	return D3D_OK;
}

ULONG BIDirect3DVertexBuffer8::AddRef()
{
	MAPIDEBUG(VertexBuffer::AddRef);
	m_refCount++;
	return D3D_OK;
}

HRESULT BIDirect3DVertexBuffer8::GetDevice(IDirect3DDevice8 **ppDevice)
{
	IAPIDEBUG(VertexBuffer::GetDevice);
	m_pDevice->AddRef();	
	*ppDevice = m_pDevice;
	return D3D_OK;
}

DWORD BIDirect3DVertexBuffer8::SetPriority(DWORD PriorityNew)
{
	APIDEBUG(VertexBuffer::SetPriority);
	return PriorityNew;
}

DWORD BIDirect3DVertexBuffer8::GetPriority()
{
	APIDEBUG(VertexBuffer::GetPriority);
	return 0;
}

D3DRESOURCETYPE BIDirect3DVertexBuffer8::GetType()
{
	APIDEBUG(VertexBuffer::GetType);
	return D3DRTYPE_VERTEXBUFFER;
}

HRESULT BIDirect3DVertexBuffer8::Lock(UINT OffsetToLock, UINT SizeToLock, BYTE **ppbData, DWORD Flags)
{
//	APIDEBUG(VertexBuffer::Lock);
	*ppbData = m_pVtxBuffer + OffsetToLock;
	return D3D_OK;
}

HRESULT BIDirect3DVertexBuffer8::Unlock()
{
//	APIDEBUG(VertexBuffer::Unlock);
	return D3D_OK;
}

HRESULT BIDirect3DVertexBuffer8::GetDesc(D3DVERTEXBUFFER_DESC *pDesc)
{
	IAPIDEBUG(VertexBuffer::GetDesc);
	pDesc->FVF = this->m_FVF;
	pDesc->Format = D3DFMT_VERTEXDATA;
	pDesc->Pool = this->m_pool;
	pDesc->Size = this->m_Length;
	pDesc->Type = D3DRTYPE_VERTEXBUFFER;
	pDesc->Usage = this->m_Usage;
	return D3D_OK;
}
