#define __RELEASE_DXGL__ "0.07"
/****************************************************************************
; *
; *     File        : entrypoint.cpp
; *
; *     Description : OpenGL entry point
; *
; *     Copyright  Realtech VR 2001
; *
; *     Permission to use, copy, modify, distribute and sell this software
; *     and its documentation for any purpose is hereby granted without fee,
; *     provided that the above copyright notice appear in all copies and
; *     that both that copyright notice and this permission notice appear
; *     in supporting documentation.  Realtech makes no representations
; *     about the suitability of this software for any purpose.
; *     It is provided "as is" without express or implied warranty.
; *
; ***************************************************************************/
#include <assert.h> // assert
#include <windows.h>
#define _CINTERFACE_HH_
#ifdef _USE_MCD
#include "mcd.h"
#else
#pragma comment(lib, "opengl32.lib")
#include <gl/gl.h>
#endif
#include "../gl_init.h"
#include "wglext.h"
#include "d3d8.h"
#include "resource.h"


#define ASSERT assert
#include <stdio.h>  // MESG
void MESG(const char *pszFormat, ...)
{
	char s[128];
	va_list va;
	va_start(va, pszFormat);
	vsprintf(s, pszFormat, va);
	va_end(va);
	OutputDebugString(s);	

	FILE *fp;
	fp = fopen("d3d8.log","a+t");
	if (fp)
	{		
		fprintf(fp,"%s",s);
		fclose(fp);
	}
}

// Win32 GL Driver
struct gl_driver_information 
{
	HWND		hwnd;
	HDC 		DC;
	HGLRC		RC;
	DEVMODE *	pmode;
	int			nmode;
	int			width;
	int			height;
};

static struct gl_driver_information *g_pseudo_device = NULL;

void GL_GetViewportSize(gl_driver_information *pDevice, int *lx, int *ly)
{
	if (lx)
		*lx = pDevice->width;

	if (ly)
		*ly = pDevice->height;
}

void GL_GetCreationParameters(struct gl_driver_information *pDevice, struct _D3DDEVICE_CREATION_PARAMETERS *p)
{
	p->AdapterOrdinal = D3DADAPTER_DEFAULT ;
	p->BehaviorFlags = D3DCREATE_PUREDEVICE | D3DCREATE_MULTITHREADED | D3DCREATE_HARDWARE_VERTEXPROCESSING ;
	p->DeviceType = D3DDEVTYPE_HAL;
	p->hFocusWindow = pDevice->hwnd;
}

// This is the default Direct3D 8 mode
void GL_DefaultSettings(int width, int height)
{
	glEnable(GL_CULL_FACE);
	glCullFace(GL_FRONT);
    
	glMatrixMode( GL_PROJECTION);
    glLoadIdentity();
	glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();

    glViewport(0, 0, width, height);
    glOrtho(0, width, height, 0, 1, -1 );	
    
    glPixelStorei( GL_UNPACK_ALIGNMENT, 1);
    glAlphaFunc( GL_GEQUAL, 0.5f);

}

// TODO: Use wglAllocateMemoryNV if available
void *glAllocateMemory(int size)
{
	return new BYTE[size];
}

// TODO: Use wglFreeMemoryNV if available
void glFreeMemory(void *ptr)
{
	delete [] ptr;
}

// Get Display mode 
void GL_GetAdapterDisplayMode(struct gl_driver_information *pDevice, UINT *width, UINT *height, UINT *bpp, UINT *refreshRate)
{
	ASSERT(pDevice);
	pDevice->DC = GetDC(pDevice->hwnd);
	
	*width		 = ::GetDeviceCaps(pDevice->DC, HORZRES);
	*height 	 = ::GetDeviceCaps(pDevice->DC, VERTRES);
	*bpp		 = ::GetDeviceCaps(pDevice->DC, BITSPIXEL);
	*refreshRate = 0; // ::GetDeviceCaps(pDevice->DC, VREFRESH); - not support by BC 6.5?

	ReleaseDC(pDevice->hwnd, pDevice->DC);
	pDevice->DC = 0;

	return;
}
// Get adapter driver name

void GL_GetAdapterName(struct gl_driver_information *pDevice, int index, char *s, int numBytes)
{
	if (pDevice->hwnd)
		strncpy(s, (char*)glGetString(GL_RENDERER), numBytes);
	else
		strncpy(s,  "OpenGL Layer Display", numBytes);
	return;
}
// Get number of display modes available

int GL_GetAdapterModeCount(struct gl_driver_information *pDevice)
{
	return pDevice->nmode;
}
// Get Adapter Count
int GL_GetAdapterCount(struct gl_driver_information *pDevice)
{
	return 1;
}

// Get adapter mode
void GL_EnumAdapterModes(struct gl_driver_information *pDevice, int index, UINT *width, UINT *height, UINT *bpp, UINT *refreshRate)
{
	DEVMODE *drv = pDevice->pmode + index;
	*width = drv->dmPelsWidth;
	*height = drv->dmPelsHeight;
	*bpp = drv->dmBitsPerPel;
	*refreshRate = drv->dmDisplayFrequency;

}

// Setup pixel format
static int SetupPixelFormat(HDC hDC, int bpp, int depth, int dlb, int acc)
{
	int pixelFormat;

	
	pixelFormat = ChoosePixelFormatEx(hDC, &bpp, &depth, NULL, &dlb, &acc, NULL);
	if (pixelFormat == 0) 
		return -1;

	PIXELFORMATDESCRIPTOR pfd;
	pfd.nSize = sizeof(pfd);

	if (SetPixelFormat(hDC, pixelFormat, &pfd) != TRUE) 
	{
		return -2;
	}
	return 0;
}

// Page flipping
void GL_PresentDevice(struct gl_driver_information *pDevice, void * hwndOverride)
{
	if (!hwndOverride)
		hwndOverride = pDevice->hwnd;

	pDevice->DC = GetDC((HWND)hwndOverride);
	SwapBuffers(pDevice->DC);
	ReleaseDC((HWND)hwndOverride, pDevice->DC);
	pDevice->DC = 0;

	return;
}

// Device creation
int GL_CreateDevice(struct gl_driver_information **pDevice, void *hwnd, int deviceId, int width, int height)
{
	gl_driver_information *p = new gl_driver_information;
	*pDevice = 0;

	if (!p)
		return -4;

	if (g_pseudo_device)
	{
		wglMakeCurrent(NULL, NULL);
		if (g_pseudo_device->RC) 
		{
			wglDeleteContext(g_pseudo_device->RC); 	
		}
		if (g_pseudo_device->DC)
		{
			ReleaseDC(g_pseudo_device->hwnd, g_pseudo_device->DC);  
		}
		if (g_pseudo_device->pmode)
		{
			delete [] g_pseudo_device->pmode;
		}
		delete g_pseudo_device;
		g_pseudo_device = 0;
	}
	else
	{
// Install MCD driver - Actually forced to ICD
#ifdef _USE_MCD
	mcdInit(NULL, TRUE, TRUE);
#endif
	}

	p->hwnd = (HWND)hwnd;
	p->DC = GetDC(p->hwnd);


	int ret = SetupPixelFormat(p->DC, 32, 32, 1, 1);
	if (ret)
	{
		delete p;
		return ret;
	}
	p->RC = wglCreateContext(p->DC);
	if (!p->RC) 
	{
		delete p;
		return -3;
	}
	wglMakeCurrent(p->DC, p->RC);	
	wglGetProcAddresses();
	wglQueryDisplayModes(&p->pmode, &p->nmode);

	*pDevice = p;

	if (p->DC)
	{
		ReleaseDC(p->hwnd, p->DC);
		p->DC = 0;
	}
	
	// API must not touch the window ...
	if (width && height)
	{
		
	}
	else
	{
		RECT rect;
		::GetWindowRect(p->hwnd, &rect);
		width = rect.right - rect.left;
		height = rect.bottom - rect.top;
	}	

	GL_DefaultSettings(width, height);	

	p->width = width;
	p->height = height;
	return 0;
}

// Create pseudo device
int GL_CreatePseudoDevice(struct gl_driver_information **pDevice)
{
	gl_driver_information *p = new gl_driver_information;
	*pDevice = 0;

	if (!p)
		return -4;
	memset(p, 0, sizeof(gl_driver_information));
	p->hwnd = (HWND)0;

	// Install MCD driver - Actually forced to ICD
#ifdef _USE_MCD
	mcdInit(NULL, TRUE, TRUE);
#endif

	wglQueryDisplayModes(&p->pmode, &p->nmode);

	*pDevice = p;
	return 0;
}



void GL_ReleaseDevice(struct gl_driver_information *pDevice)
{
	wglMakeCurrent(NULL, NULL);
	if (pDevice)
	{
		if (pDevice->RC) 
		{
			wglDeleteContext(pDevice->RC); 	
			pDevice->RC = 0;
		}
		if (pDevice->DC)
		{
			ReleaseDC(pDevice->hwnd, pDevice->DC);  
			pDevice->DC = 0;
		}
		if (pDevice->pmode)
		{
			delete [] pDevice->pmode;
			pDevice->pmode = 0;
		}
		delete pDevice;
	}
#ifdef _USE_MCD
	mcdRelease();
#endif
	return;
}

HINSTANCE hInstance;

BOOL APIENTRY DllMain( HINSTANCE hModule, 
					   DWORD  ul_reason_for_call, 
					   LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:		
			hInstance = hModule;
			break;
		case DLL_THREAD_ATTACH:		

		case DLL_THREAD_DETACH:
			break;
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}

bool bBoot;
#if D3D_SDK_VERSION >= 220	
#define EMU_SDK "Direct3D 8.1"
#else
#define EMU_SDK "Direct3D 8.0"
#endif
static BOOL WINAPI bootDialog(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{	
	switch (message) 
	{
		case WM_TIMER:
		EndDialog(hWnd, 1);
		break;
		case WM_CLOSE:
		case WM_DESTROY:
		EndDialog(hWnd, 1);
		break;		  
		case WM_INITDIALOG: 		
			::SetDlgItemText(hWnd, IDC_TEXT, "DXGL Win32/Wrapper "__RELEASE_DXGL__" - "EMU_SDK"\nBuild "__DATE__"\nhttp://www.realtech-vr.com/directx");
			ShowWindow(hWnd, TRUE); 		
			break;	
	}
	return FALSE;
}
const char *GL_GetErrorString(GLenum err)
{
	switch(err)
	{
		case GL_NO_ERROR: return "NO ERROR"; 
		case GL_INVALID_ENUM: return "INVALID ENUM"; 
		case GL_INVALID_OPERATION: return "INVALID OPERATION"; 		
		case GL_STACK_UNDERFLOW: return "STACK UNDERFLOW"; 
		case GL_STACK_OVERFLOW: return "STACK OVERFLOW"; 
		case GL_OUT_OF_MEMORY: return "OUT OF MEMORY"; 
	}
	return "UNDEFINED";
}


#include "resource.h"

void DisplaySplashScreen(int sdk)
{
	if (!bBoot)
	{
		bBoot = true;
		HWND h = ::CreateDialog(hInstance, MAKEINTRESOURCE(IDD_BOOT), NULL, (DLGPROC) bootDialog);
		if (h)
		{
			::ShowWindow(h, TRUE);
			::UpdateWindow(h);
			unsigned t = ::GetTickCount()+ 1500;
			while (GetTickCount()<t)
			{
			}
			::DestroyWindow(h);
		}
	}
}
